package convert.buffers;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.Buffer;

/**
 * Class used as a producer: reads strings from the console and puts them in a
 * "buffer". The buffer in this instance is an ASF Commons Collection
 * implementation of the Buffer interface.
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class StringProducer {
    /** Expected "command" to exit program. */
    public static final String EXIT_CMD = "q";

    /** Buffer used internally to dump the objects produced to. */
    private Buffer             buffer;

    /**
     * Used to read strings from console. The strings are deposited in
     * {@link #queue the queue} and this is how they are produced.
     */
    private BufferedReader     reader;

    /** The producing of strings is done in a separate thread -- this one. */
    private Thread             producerThread;

    /** Notifies these component when we're done. */
    private List<EndListener>  listeners;

    /**
     * Creates an object and initializes the queue we will be producing the
     * objects into.
     *
     * @param buffer
     *            Buffer which will store all the produced objects
     */
    public StringProducer(Buffer buffer) {
        this.buffer = buffer;
        this.reader = new BufferedReader(new InputStreamReader(System.in));
        this.listeners = new ArrayList<EndListener>();
        this.producerThread = new Thread(new Runnable() {
            @SuppressWarnings("unchecked")
            @Override
            public void run() {
                String line;
                try {
                    while ((line = reader.readLine()) != null) {
                        if (EXIT_CMD.equalsIgnoreCase(line)) {
                            break;
                        }
                        try {
                            StringProducer.this.buffer.add(line);
                        } catch (IllegalArgumentException e) {
                            // not an integer
                        }
                    }
                } catch (IOException e) {
                    System.err.println("Error:" + e.getMessage());
                } finally {
                    notifyDone();
                }
            }
        });
    }

    public void addEndListener(EndListener listener) {
        listeners.add(listener);
    }

    public void startProduce() {
        producerThread.start();
    }

    private void notifyDone() {
        for (EndListener l : listeners) {
            l.end();
        }
    }
}
