package convert.oop;

import java.util.concurrent.BlockingQueue;

/**
 * Simple consumer. Takes a string from a given "buffer" (queue) and transforms
 * it into an integer and then prints it on console.
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class StringConsumer implements EndListener {
    /** Queue used to consume data from. */
    private BlockingQueue<String>        queue;

    /** The transform operation used to convert strings into integers. */
    private Transformer<String, Integer> transformer;

    /** Consumption of strings is done in this separate thread. */
    private Thread                       consumerThread;

    /**
     * Creates a consumer which starts right away consuming items from the
     * queue. The {@link #consumerThread consumer thread} runs until
     * {@link #end() the end notification} is received.
     *
     * @param queue
     *            Queue to consume from
     * @param transformer
     *            Implementation of an operation which converts strings to
     *            integers
     */
    public StringConsumer(BlockingQueue<String> queue, Transformer<String, Integer> transformer) {
        this.queue = queue;
        this.transformer = transformer;
        this.consumerThread = new Thread(new Runnable() {
            @Override
            public void run() {
                while (!StringConsumer.this.consumerThread.isInterrupted()) {
                    try {
                        String s = StringConsumer.this.queue.take();
                        Integer i = StringConsumer.this.transformer.transform(s);
                        System.out.println(i);
                    } catch (InterruptedException e) {
                        break;
                    }
                }
            }
        });
        this.consumerThread.start();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void end() {
        consumerThread.interrupt();
    }
}
