package convert.oop;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Queue;

/**
 * Class used as a producer: reads strings from the console and puts them in a
 * "buffer". The buffer is a simple queue in this case, which is supplied at
 * creation time. Please note that this class is <b>not</b> synchronized, so if
 * this is needed, make sure the queue supplied in is synchronized.
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class StringProducer {
    /** Expected "command" to exit program. */
    public static final String EXIT_CMD = "q";

    /** Queue used internally to dump the objects produced to. */
    private Queue<String>      queue;

    /**
     * Used to read strings from console. The strings are deposited in
     * {@link #queue the queue} and this is how they are produced.
     */
    private BufferedReader     reader;

    /**
     * The producing of strings is done in a separate thread -- this one.
     */
    private Thread             producerThread;

    /** Notifies these component when we're done. */
    private List<EndListener>  listeners;

    /**
     * Creates an object and initializes the queue we will be producing the
     * objects into.
     *
     * @param queue
     *            Queue which will store all the produced objects
     */
    public StringProducer(Queue<String> queue) {
        this.queue = queue;
        this.reader = new BufferedReader(new InputStreamReader(System.in));
        this.listeners = new ArrayList<EndListener>();
        this.producerThread = new Thread(new Runnable() {
            @Override
            public void run() {
                String line;
                try {
                    while ((line = reader.readLine()) != null) {
                        if (EXIT_CMD.equalsIgnoreCase(line)) {
                            break;
                        }
                        StringProducer.this.queue.add(line);
                    }
                } catch (IOException e) {
                    System.err.println("Error:" + e.getMessage());
                } finally {
                    notifyDone();
                }
            }
        });
    }

    public void addEndListener(EndListener listener) {
        listeners.add(listener);
    }

    public void startProduce() {
        producerThread.start();
    }

    private void notifyDone() {
        for (EndListener l : listeners) {
            l.end();
        }
    }
}
